#!/bin/sh 
#############################################################
#
# Ir Slice Creator v1.0.01
#
# written by Brad McGill <mcgillpub@iinet_dot_net_dot_au>
# Developed for use by the OzTivo Community during April 2005
# A guide to its use can be found here : <coming soon>
#
#############################################################
#
# Version History
# ---------------
#
# v1.0.01 - Now delete $key.cap before capturing code, if
#           serialPortArbitrator (lower case s) then create a link
#           to Serial PortArbitrator.  Also fixed kill of irLock.
#
# v1.0.01 - Removed use of sed by stripping spaces using in 
#           built string manipulation, this is a much better 
#           approach, also fixed a few small bugs.
#
# v0.9b Beta Release - Mostly Done... Needs some testing and
#                      and still need to implement load 
#                      device file to tivo, this ones hard to 
#                      test on my only tivo.
#
#############################################################


########################
# Declare Variables
########################
if [[ "$1" = "--help" || "$1" = "--?" || "$1" = "-h" ]]
then
	echo "Usage: $0 <frequency> <retries> <targetlength> <deviceCode> <deviceName>"
	echo "or just $0 with no parameters to configure these in the menu."
	echo ""
	exit 65
fi

frequency=$1
retries=${2-5}
targetLen=$3
devCode=$4
devName=$5

resultKey="" #used as return result from choosekey methods
foundKeys="" #list of keys found so far



########################
# YesNo - returns 0 or 1
########################
yes_no()
{
	while [ 1 ]
	do
		echo "$1 [Y/N]"
		read
    response=$REPLY
		case "$response" in
		"Y" | "y")
			return 0
		;;
		"N" | "n")
			return 1
		;;
		esac
	done
	
}

########################
# ChooseKey - returns key selected
########################
choose_key()
{
	PS3='Select key to capture: '
	select chooseKey in 1 2 3 4 5 6 7 8 9 0 Enter
	do
		resultKey=$chooseKey
		break
	done
}

########################
# getCapturedKeys
########################
get_captured_keys ()
{
	keyCount=0
	foundKeys=""	
	for findKey in 1 2 3 4 5 6 7 8 9 0 Enter
	do
		if [ -e key$findKey.code ]
		then
			foundKeys="$foundKeys $findKey"
			keyCount=$((keyCount+1))
		fi
	done
	return $keyCount

}

########################
# ChooseCapturedKey - returns key selected
########################
choose_captured_key()
{
	PS3='Select key to transmit: '
	get_captured_keys
	select choosekey in $foundKeys
	do
		resultKey=$choosekey
		break
	done
}

####################
# determineLength
####################
determine_target_length () 
{
	./irlock /sbin/irtest -t /dev/ttyS0 -l > calcLength.cap &

	code=1
	lastCount=0
	count=0
	totalLen=0

	echo "Push one remote button at each prompt determination code length:"

	while [ $code -le $retries ]
	do
		echo "Push same button again"
		#wait for button push
		while [ $lastCount -eq $count ]
		do
			sleep 1
			count=`grep -G "^[0123456789 ]" calcLength.cap | wc | cut -c1-7`
			count=${count## * } #Strip spaces from start of string
		done

		let "codeLen=$count - $lastCount"

		echo "Got data length $codeLen. $code successful captures. Captures Required:$retries"
		lastCount=$count
		let "totalLen=$totalLen + $codeLen"

		code=$((code+1))
	done
	code=$((code-1))
	let "targetLen=$totalLen / $code"
	echo "Got Target Length of $targetLen"
	
	#pid=`ps | grep "00 /sbin/irtest" | cut -c1-5`
  pid=`ps u | grep -e "00 /sbin/irtest" | cut -c7-14`
	pid=${pid## * } #Strip spaces from start of string
	kill -9 $pid
}


####################
# determine_remote_frequency
####################
determine_frequency ()
{
	#Build a remote code for each freq
	get_captured_keys
	for oneKey in $foundKeys
	do
		irCode=`cat key$oneKey.code | cut -d\" -f4`
		shortCode=`echo "$irCode" | cut -d" " -f3-`

		for testFreq in 31 33 43 45 47 48 49 50 67
		do
			testCode="0 $testFreq $shortCode"
			while [ 1 ]
			do
				echo "About to transmit $oneKey at $testFreq Khz"
				./irlock /sbin/irtest -t /dev/ttyS0 -- $testCode > /dev/null
				yes_no "Retransmit Code"				
				if [ $? -eq 1 ]
				then
					break
				fi
			done			
		done
		echo "Now select the frequency which worked best:"
		echo "Frequency [31,33,43,45,47,48,49,50,67] $frequency:"
		read
    response=$REPLY
		if [ -n $response ]
		then
			frequency=$response
		fi
		break #only need to do it for one key
	done
}

####################
# CaptureCode
####################
capture_code()
{
	# Init Variables
	key=$1
	code=1
	lastCount=0
	count=0

	# Start Capture
	./irlock /sbin/irtest -t /dev/ttyS0 -l > $keyRaw.cap &

  # Delete the key.cap file if it exists
  if [ -e $key.cap ]
     then
     rm $key.cap
  fi

	# Capture repetitions
	echo "Ready to capture key $key:"
	while [ $code -le $retries ]
	do
		#wait for button push
		while [ $lastCount -eq $count ]
		do
			sleep 1
			count=`grep -G "^[0123456789 ]" $keyRaw.cap | wc | cut -c1-7`
			count=${count## * } #Strip spaces from start of string			
		done

		let "codeLen=$count - $lastCount"

		if [ $codeLen -eq $targetLen ]
		then
			#Good length code, use tail to copy to file new file
			echo "Got good code of length $codeLen. Good Codes:$code Codes Required:$retries"
			code=$((code+1))
			grep -G "^[0123456789 ]" $keyRaw.cap | tail -n$codeLen >> $key.cap
			lastCount=$count
		else
			#Bad Code, try again copy nothing to new file
			echo "Got bad length of $codeLen, wanted: $targetLen"
			lastCount=$count
		fi
	done

	# Kill capture 
  #ps u | grep -e "00 /sbin/irtest"
	pid=`ps u | grep -e "00 /sbin/irtest" | cut -c7-14`
	pid=${pid## * } #Strip spaces from start of string
  #echo "pid=" $pid
	kill -9 $pid

	# Output Results
	echo "Converting code..."
	irCode="`tivosh irtest2tivo.tcl $key.cap $frequency $targetLen`"

	echo "Storing code..."
	echo "setIr \$tivoir \"Dig$key\" \"$irCode\"" > key$key.code

}

########################
# test_code
########################
test_code ()
{
	while [ 1 ]
	do
		choose_captured_key
		echo "Trasmitting $resultKey"
		./irlock /sbin/irtest -t /dev/ttyS0 -- `cat key$resultKey.code | cut -d\" -f4`  >/dev/null
		yes_no "Transmit another key"
		if [ $? -eq 1 ]
		then
			break
		fi
	done
	
}

########################
# output_device_tcl
########################
output_device_tcl ()
{
	echo "Outputting all captured codes to $devName.tcl"
	echo ""
	echo "source /tvlib/tcl/tv/mfslib.tcl" 				>$devName.tcl
	echo "" 							>>$devName.tcl
	echo "proc setIr { irObj field irSequence } {" 			>>$devName.tcl
	echo "  set savedInitial 0" 					>>$devName.tcl
	echo "" 							>>$devName.tcl
	echo "  foreach irPart \$irSequence {" 				>>$devName.tcl
	echo "" 							>>$devName.tcl
	echo "    if {\$savedInitial == 0} {" 				>>$devName.tcl
	echo "      set savedInitial 1" 				>>$devName.tcl
	echo "      dbobj \$irObj set \$field \$irPart" 		>>$devName.tcl
	echo "    } else {" 						>>$devName.tcl
	echo "      dbobj \$irObj add \$field \$irPart" 		>>$devName.tcl
	echo "    }" 							>>$devName.tcl
	echo "  }" 							>>$devName.tcl
	echo "}" 							>>$devName.tcl
	echo "" 							>>$devName.tcl
	echo "proc $devName {} {"					>>$devName.tcl
	echo "" 							>>$devName.tcl
	echo "  set db [dbopen]" 					>>$devName.tcl
	echo "    " 							>>$devName.tcl
	echo "  puts \"Setting $devName codes for TiVo $devCode...\"" 	>>$devName.tcl
	echo "" 							>>$devName.tcl
	echo "  RetryTransaction {" 					>>$devName.tcl
	echo "" 							>>$devName.tcl
	echo "  set tivoir [db \$db open /Component/Ir/TivoFormat/$devCode]" >>$devName.tcl
	echo "" 							>>$devName.tcl
	
	get_captured_keys
	for outKey in $foundKeys
	do
		cat key$outKey.code 					>> $devName.tcl
	done
	
	echo "    }" 							>>$devName.tcl
	echo "    puts \"Finished! Reboot your TiVo for the new codes to take effect.\"" >>$devName.tcl
	echo "    " 							>>$devName.tcl
	echo "    dbclose \$db" 					>>$devName.tcl
	echo "}" 							>>$devName.tcl
	echo "" 							>>$devName.tcl
}

########################
# output_transmitkey
########################
output_transmitkey ()
{
	echo ""
	echo "Outputting code $resultKey to transmit$resultKey.sh"
	echo ""
	echo "#!/bin/sh" >transmit$resultKey.sh
	echo "echo \"Transmitting $resultKey\"" >>transmit$resultKey.sh
	code=`cat key$resultKey.code | cut -d\" -f4`
	echo "./irlock /sbin/irtest -t /dev/ttyS0 -- $code >/dev/null" >>transmit$resultKey.sh
	echo "" >>transmit$resultKey.sh
	chmod 755 transmit$resultKey.sh
}

########################
# set_options
########################
set_options()
{
	echo "Set Options"
	echo "==========="
	echo "Current value is shown after possible options in [] brackets."
	echo "Just hit enter to keep current value."
	echo ""

	echo "Frequency [31,33,43,45,47,48,49,50,67] $frequency:"
	read
  response=$REPLY
	if [ -n "$response" ]
	then
		frequency=$response
	fi

	echo "Code Length [1,2,3] $targetLen:"
	read
  response=$REPLY
	if [ -n "$response" ]
	then
		targetLen=$response
	fi

	echo "Device Code [29992-29999] $devCode:"
	read
  response=$REPLY
	if [ -n "$response" ]
	then
		devCode=$response
	fi

	echo "Device Name [string] $devName:"
	read
  response=$REPLY
	if [ -n "$response" ]
	then
		devName=$response
	fi

	echo "Repetitions [3-20] $retries:"
	read
  response=$REPLY
	if [ -n "$response" ]
	then
		retries=$response
	fi

}

########################
# set_freq_on_codes
########################
set_freq_on_codes() {
	for findKey in 1 2 3 4 5 6 7 8 9 0 Enter
	do
		if [ -e key$findKey.code ]
		then
			#change frequency on file
			echo "Converting code $findKey..."
			irCode=`cat key$findKey.code | cut -d\" -f4`
			shortCode=`echo "$irCode" | cut -d" " -f3-`

			outCode="0 $frequency $shortCode"
			echo "setIr \$tivoir \"Dig$findKey\" \"$outCode\"" > key$findKey.code
		fi
	done
	echo "Conversion done."
}


########################
# Display Menu
########################
display_menu() 
{
	get_captured_keys
	echo "Ir Slice Creator v0.9b by Brad McGill"
	echo ""
	echo "Frequency: $frequency"
	echo "Code Length: $targetLen"
	echo "Device Code: $devCode"
	echo "Device Name: $devName"
	echo "Repetitions: $retries"
	echo "Keys Captured: $foundKeys"
	echo ""
	echo "0. Options"
	echo "1. Determine target code length"
	echo "2. Determine Frequency of Remote"
	echo "3. Capture one key"
	echo "4. Capture all keys"
	echo "5. Capture uncaptured keys"
	echo "6. Test Remote Codes"
	echo "7. Output Codes to Device File"
	echo "8. Output One code to a TramsitKey.sh script"
	echo "9. Load Device File to Tivo"
	echo "a. Set all captured codes to current frequency"
	echo "x. Exit"
}

########################
# Main Method
########################
error () 
{
	echo "******"
	echo "Error:"
	echo "$1"
	echo "******"
}

########################
# Main Method
########################
# Create a link to SerialPortArbitrator if necessary
if [ ! -e /var/tmp/SerialPortArbitrator ]
   then
   ln -s /var/tmp/serialPortArbitrator  /var/tmp/SerialPortArbitrator
fi

# Clear out any rogue irlocks
pid=`ps u | grep -e "00 /sbin/irtest" | cut -c7-14`
pid=${pid## * } #Strip spaces from start of string
#echo "pid="$pid
#echo "pid len="${#pid}
if [ ${#pid} -ne 0 ]
  then
  kill -9 $pid
fi

while [[ $choice != "x" ]]
do
	display_menu
	read
  choice=$REPLY
	case "$choice" in
	"1")
		determine_target_length
	;;
	"2")
		get_captured_keys
		if [ $? -eq 0 ]
		then
			error "You must capture at least one code before you can determine the remote's frequency." 
		else
			determine_frequency
		fi
	;;
	"3") #capture one key
		if [[ -z "$frequency" || -z "$targetLen" || -z "$retries" ]]
		then
			error "Frequency, TragetLength and Retries must be set to capture codes."
		else
			choose_key
			capture_code $resultKey 
		fi
	;;
	"4") #capture all keys
		if [[ -z "$frequency" || -z "$targetLen" || -z "$retries" ]]
		then
			error "Frequency, TragetLength and Retries must be set to capture codes."
		else
			yes_no "Capture Enter key"
			if [ $? -eq 0 ]
			then
				keyList="0 1 2 3 4 5 6 7 8 9 Enter"
			else
				keyList="0 1 2 3 4 5 6 7 8 9"
			fi
			for loopKey in $keyList
			do
				capture_code $loopKey 
			done
		fi
	;;
	"5") #Capture Uncaptured Keys
		if [[ -z "$frequency" || -z "$targetLen" || -z "$retries" ]]
		then
			error "Frequency, TragetLength and Retries must be set to capture codes."
		else
			yes_no "Capture Enter key"
			if [ $? -eq 0 ]
			then
				keyList="0 1 2 3 4 5 6 7 8 9 Enter"
			else
				keyList="0 1 2 3 4 5 6 7 8 9"
			fi
			for loopKey in $keyList
			do
				if [ ! -e key$loopKey.code ]
				then
					capture_code $loopKey
				fi
			done
		fi
	;;
	"6")		
		get_captured_keys
		if [ $? -eq 0 ]
		then
			error "You must capture at least one code before you can test them." 
		else
			test_code 
		fi
		
	;;
	"7")
		yes_no "Does your device use an Enter key"
		if [ $? -eq 0 ]
		then
			get_captured_keys
			if [ "$foundKeys" = " 1 2 3 4 5 6 7 8 9 0 Enter" ]
			then
				output_device_tcl
			else
				error "You must capture the keys 0 1 2 3 4 5 6 7 8 9 Enter before you can output your device file"
			fi
		else
			get_captured_keys
			if [[ "$foundKeys" = " 1 2 3 4 5 6 7 8 9 0" || "$foundKeys" = " 1 2 3 4 5 6 7 8 9 0 Enter" ]]
			then
				output_device_tcl
			else
				error "You must capture the keys 0 1 2 3 4 5 6 7 8 9 before you can output your device file"
			fi
		fi
	;;
	"8")
		get_captured_keys
		if [ $? -eq 0 ]
		then
			error "You must capture at least one code before ouput a trasmitkey file." 
		else
			choose_captured_key
			output_transmitkey
		fi
	;;
	"9")    
    error "This option not yet implemented"
    #echo "This option not yet implemented"
    #echo "Type in the following commands:"
    #echo "tivosh"
    #echo "%source test.tcl"
    #test
    #exit
		#load tcl to tivo
		#if [ ! -e $devName.tcl ]
		#then
		#	output_device_tcl
		#fi
		#tivosh $devName.tcl
	;;
	"0")
		set_options
	;;
	"a")
		set_freq_on_codes
	;;
	esac
done
